#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#include "misc.h"
#include "empthread.h"

#include <varargs.h>

  
#ifdef _EMPTH_POSIX
static pthread_key_t ctx_key;
static int empth_flags;
static char **udata; /* pointer to out global context */

static pthread_mutex_t mtx_ctxsw;  /* thread in critical section */


static void empth_status();
static void empth_setctx _PROTO((void *));
static void empth_restorectx _PROTO(());

void *empth_start _PROTO((void *));

void *empth_start(ctx)
void *ctx;
{
    struct sigaction act;
    extern emp_sig_t panic();
    extern emp_sig_t shutdwn();

    
    /* actually it should inherit all this from main but... */ 
#ifdef SA_SIGINFO
    act.sa_flags = SA_SIGINFO;
#endif
    sigemptyset (&act.sa_mask);
    act.sa_handler = shutdwn;
    sigaction (SIGUSR1, &act, NULL);
    sigaction (SIGTERM, &act, NULL);
    sigaction (SIGINT, &act, NULL);
    act.sa_handler = panic;
    sigaction (SIGBUS, &act, NULL);
    sigaction (SIGSEGV, &act, NULL);
    sigaction (SIGILL, &act, NULL);
    sigaction (SIGFPE, &act, NULL);
    act.sa_handler = SIG_IGN;
    sigaction(SIGPIPE, &act, NULL);

    act.sa_handler = empth_alarm;
    sigaction(SIGALRM, &act, NULL);

    ((empth_t *)ctx)->id = pthread_self();
    pthread_setspecific(ctx_key, ctx);
    pthread_mutex_lock(&mtx_ctxsw);
    *udata = ((empth_t *)ctx)->ud;
    ((empth_t *)ctx)->ep(((empth_t *)ctx)->ud);
    empth_exit();
}    
    
static void
empth_status(va_alist)
	va_dcl
{
	va_list	ap;
	char	*format;
	static	struct timeval startTime;
	struct	timeval tv;
	char	buf[1024];
	int	sec, msec;
	empth_t *a;
	
	va_start(ap);
	format = va_arg(ap, char *);
	if (empth_flags & EMPTH_PRINT) {
		if (startTime.tv_sec == 0)
			gettimeofday(&startTime, 0);
		gettimeofday(&tv, 0);
		sec = tv.tv_sec - startTime.tv_sec;
		msec = (tv.tv_usec - startTime.tv_usec) / 1000;
		if (msec < 0) {
			sec++;
			msec += 1000;
		}
		vsprintf(buf, format, ap);
	        a = empth_self();
		printf("%d:%02d.%03d %17s: %s\n", sec/60, sec%60, msec/10,
		       a->name,
		       buf);

	}
	va_end(ap);
}


int
empth_init (ctx_ptr, flags)
char **ctx_ptr;
int flags;
{
    empth_t *ctx;
    struct sigaction act;
    

    pthread_key_create(&ctx_key, 0);    
#ifdef _DECTHREADS_
    pthread_mutex_init(&mtx_ctxsw, pthread_mutexattr_default);
#else
    pthread_mutex_init(&mtx_ctxsw, 0);
#endif
    
    act.sa_flags = 0;
    sigemptyset (&act.sa_mask);
    act.sa_handler = empth_alarm;
    sigaction(SIGALRM, &act, NULL);
    
    udata = ctx_ptr;
    ctx = (empth_t *)malloc(sizeof(empth_t));
    if(!ctx) {
	logerror("pthread init failed: not enough memory");
	exit(1);
    }
    ctx->name = "Main";
    ctx->desc = "empire main";
    ctx->ep = 0;
    ctx->ud = 0;
    ctx->id = pthread_self();
    ctx->state = 0;	
    pthread_setspecific(ctx_key, ctx);
    pthread_mutex_lock(&mtx_ctxsw);    
    empth_flags = flags;
    logerror("pthreads initialized");
}

    
/*
 * prio can be used for setting scheeduling policy but...
 * it seems to be optional in POSIX threads and Solaris
 * for example just ignores it.
 * More then that priority is not needed even in lwp threads.
 */
empth_t *
empth_create (prio, entry, size, flags, name, desc, ud)
int prio;
void (*entry)();
int size;
int flags;
char *name;
char *desc; 
void *ud;
{
    pthread_t t;
    pthread_attr_t attr;
    empth_t *ctx;
    
    empth_status("creating new thread %s:%s", name, desc);
    
    ctx = (empth_t *)malloc(sizeof(empth_t));
    if(!ctx) {
	logerror("not enough memoty to create thread: %s (%s)", name, desc);
	return NULL;
    }
    ctx->name = strdup(name);
    ctx->desc = strdup(desc);
    ctx->ud = ud;
    ctx->state = 0;	    
    ctx->ep = entry;

    if(pthread_attr_init(&attr)) {
	logerror("can not create thread attribute %s (%s): %s", name, desc,
		 strerror(errno));
	goto bad;
    }

#ifdef _DECTHREADS_
    /* DEC does not have PTHREAD_STACK_MIN constant */
    /* Do not go below default size                 */
    if(size > pthread_attr_getstacksize(attr))
      pthread_attr_setstacksize(&attr, size);
#else
    if(size < PTHREAD_STACK_MIN)
      size = PTHREAD_STACK_MIN + 1;

    pthread_attr_setstacksize(&attr, size);
#endif
    
#ifdef _DECTHREADS_    
    if(pthread_create(&t, attr, empth_start, (void *)ctx)) {
#else
    if(pthread_create(&t, &attr, empth_start, (void *)ctx)) {
#endif
	
      logerror("can not create thread: %s (%s): %s", name, desc,
	       strerror(errno));
      goto bad;
    }
    empth_status("new thread id is %d", t);
    return ctx;
    pthread_attr_destroy(&attr);
bad:
    pthread_attr_destroy(&attr);
    free(ctx);
    return NULL;
}



static void
empth_setctx(ct)
void *ct;
{
    empth_t *ctx_ptr;
#ifdef _DECTHREADS_
    pthread_getspecific(ctx_key, (pthread_addr_t *)&ctx_ptr);
#else
    ctx_ptr = (empth_t *)pthread_getspecific(ctx_key);
#endif
    ctx_ptr->ud = ct;
    *udata = ((empth_t *)ctx_ptr)->ud;
    pthread_setspecific(ctx_key, (void *)ctx_ptr);
    empth_status("context saved");
}

static void 
empth_restorectx()
{
    empth_t *ctx_ptr;
    
#ifdef _DECTHREADS_
    pthread_getspecific(ctx_key, (pthread_addr_t *)&ctx_ptr);
#else
    ctx_ptr = (empth_t *)pthread_getspecific(ctx_key);
#endif
    *udata = (char *)ctx_ptr->ud;
    if (ctx_ptr->state == EMPTH_KILLED) {
	empth_status("i am dead");
	empth_exit();
    }
    empth_status("context restored");
}

empth_t *
empth_self()
{
  empth_t *ctx_ptr;
  
#ifdef _DECTHREADS_
  pthread_getspecific(ctx_key, (pthread_addr_t *)&ctx_ptr);
  return ctx_ptr;
#else
  return (empth_t *)pthread_getspecific(ctx_key);
#endif
}

void empth_exit ()
{
    empth_t *ctx_ptr;

    pthread_mutex_unlock(&mtx_ctxsw);
    empth_status("empth_exit");
#ifdef _DECTHREADS_
    /* a ugly kludge to fix bug in pthreads on OSF. For some reason
       main can not call pthread_exit without crashing the program */
    pthread_getspecific(ctx_key, (pthread_addr_t *)&ctx_ptr);
    if(!strcmp(ctx_ptr->name, "Main")) {
	while(1) {
		pthread_yield();
		sleep(60);
	}
    }
#else
    ctx_ptr = (empth_t *)pthread_getspecific(ctx_key);
#endif
    free(ctx_ptr);
    pthread_exit(0);
}
	
void empth_yield ()
{
    pthread_mutex_unlock(&mtx_ctxsw);
    sleep(10); /* take a nap  pthread_yield(); */
    pthread_mutex_lock(&mtx_ctxsw);
    empth_restorectx();
}

void empth_terminate(a)
empth_t *a;
{
    /* logerror("calling non supported function empth_terminate: %s:%d",
	     __FILE__, __LINE__); */
    empth_status("killing thread %s", a->name);
    a->state = EMPTH_KILLED;
#ifndef _DECTHREADS_
    /* DEC has no pthread_kill. Not sure that cancel is correct. */
    pthread_kill(a->id, SIGALRM);
#endif
    return; 
}

void empth_select(fd, flags)
int fd;
int flags;
{

    fd_set readmask;
    fd_set writemask;
    struct lwpProc *proc;
    struct timeval tv;
    int n;
    
    pthread_mutex_unlock(&mtx_ctxsw);
    empth_status("%s select on %d",
		 flags == EMPTH_FD_READ ? "read" : "write", fd );
    while (1) {
	tv.tv_sec = 1000000;
	tv.tv_usec = 0;

	FD_ZERO(&readmask);
	FD_ZERO(&writemask);
	
	switch(flags) {
	case EMPTH_FD_READ:
	    FD_SET(fd, &readmask);
	    break;
	case EMPTH_FD_WRITE:
	    FD_SET(fd, &writemask);
	    break;
	default:
	    logerror("bad flag %d passed to empth_select", flags);
	    empth_exit();
	}

	n = select(fd + 1, &readmask, &writemask, (fd_set *)0, &tv);
	
	if (n < 0) {
	    if (errno == EINTR) {
		/* go handle the signal */
		empth_status("select broken by signal");
		goto done;
		return;
	    }
	    /* strange but we dont get EINTR on select broken by signal */
	    empth_status("select failed (%s)", strerror(errno));
	    goto done; 
	    return;
	}
	
	if (flags == EMPTH_FD_READ && FD_ISSET(fd, &readmask)) {
	    empth_status("input ready");
	    break;
	}
	if (flags == EMPTH_FD_WRITE && FD_ISSET(fd, &writemask)) {
	    empth_status("output ready");
	    break;
	}
    }
    
done:
    pthread_mutex_lock(&mtx_ctxsw);
    empth_restorectx();
	
}


emp_sig_t
empth_alarm(sig)
{
    struct sigaction act;
    empth_status("got alarm signal");
#ifdef SA_RESTART
    act.sa_flags &= ~SA_RESTART;
#endif
    sigemptyset (&act.sa_mask);
    act.sa_handler = empth_alarm;
    sigaction(SIGALRM, &act, NULL);
}

void empth_wakeup(a)
empth_t *a;
{
    empth_status("waking up thread %s", a->name);
#ifndef _DECTHREADS_
    pthread_kill(a->id, SIGALRM);
#endif
    empth_status("waiting for it to run");
    /* empth_yield(); */
}

void empth_sleep(until)
long until;
{
    empth_status("going to sleep %ld sec", until - time(0));
    pthread_mutex_unlock(&mtx_ctxsw);
    sleep((unsigned)(until - (long)time(0)));
    empth_status("sleep done. Waiting for lock");
    pthread_mutex_lock(&mtx_ctxsw);
    empth_restorectx();
}


empth_sem_t *empth_sem_create(name, cnt)
char *name;
int cnt;
{
    empth_sem_t *sm;

    sm = (empth_sem_t *)malloc(sizeof(empth_sem_t));
    if(!sm) {
	logerror("out of memory at %s:%d", __FILE__, __LINE__);
	return NULL;
    }
    strncpy(sm->name, name, sizeof(sm->name)-1);
    sm->count = cnt;
#ifdef _DECTHREADS_
    pthread_mutex_init(&sm->mtx_update, pthread_mutexattr_default);
    pthread_mutex_init(&sm->mtx_sem, pthread_mutexattr_default);
    pthread_cond_init(&sm->cnd_sem, pthread_condattr_default);
#else
    pthread_mutex_init(&sm->mtx_update, 0);
    pthread_mutex_init(&sm->mtx_sem, 0);
    pthread_cond_init(&sm->cnd_sem, 0);
#endif
    return sm;
}

void empth_sem_signal(sm)
empth_sem_t *sm;
{
    empth_status("signal on semaphore %s:%d", sm->name, sm->count);
    pthread_mutex_lock(&sm->mtx_update);
    if(sm->count++ < 0) {
	pthread_mutex_unlock(&sm->mtx_update);
	pthread_mutex_lock(&sm->mtx_sem);
	pthread_cond_signal(&sm->cnd_sem);
	pthread_mutex_unlock(&sm->mtx_sem);
    }
    else
	pthread_mutex_unlock(&sm->mtx_update);
}

void empth_sem_wait (sm)
empth_sem_t *sm;
{
    empth_status("wait on semaphore %s:%d", sm->name, sm->count);
    pthread_mutex_lock(&sm->mtx_update);
    if(--sm->count < 0) {
	pthread_mutex_unlock(&sm->mtx_update);
	empth_status("blocking");
	pthread_mutex_unlock(&mtx_ctxsw);
	pthread_cond_wait(&sm->cnd_sem, &sm->mtx_sem);
	empth_status("waking up");
	pthread_mutex_unlock(&sm->mtx_sem);
	pthread_mutex_lock(&mtx_ctxsw);
	empth_restorectx();
    }
    else
	pthread_mutex_unlock(&sm->mtx_update);
}

#endif







