#include <struct.h>
#include <assert.h>
#include <stdio.h>
#include <ctype.h>
#include <malloc.h>
#include "edbproc.h"
#include "edbpriv.h"	/* private structures */
#include "misc.h"
#include "file.h"
#include "xy.h"
#include "nsc.h"
#include "edb.h"

#define DIAG(x) 

#define FIELDADDR(x,y) ( ((char *) x) + y )

/* find object with no keys */
struct genobject *find_object_nokey(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{								
	/* assume that object is not present */
	return NULL;
}

/* store object with no keys */
void store_object_nokey(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{
	/* always add object to collection */
	ObjectDescriptor *this;
	Collection **coll;
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	if (!*coll) {
		*coll = newCollection(20,20);
	}
	DIAG( printf("Store %s()\n", this->objdescr); )
	addCollectionItem(*coll, rec);
}

/* find object with UCHAR keys */
struct genobject *find_object_uchar(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{								
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	u_char key1, key2;
	int slot;						
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((u_char *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((u_char *) FIELDADDR(rec, this->key2));
	if (!*coll) return NULL; 				
	DIAG( printf("Find %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=findIndexEntry(*index, key1, key2);		
	if (slot<0) return NULL;				
	return getCollectionItem(*coll, slot);		
}

/* store object with UCHAR keys */
void store_object_uchar(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	u_char key1, key2;
	int slot;
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((u_char *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((u_char *) FIELDADDR(rec, this->key2));
	if (!*coll) {
		*coll = newCollection(20,20);
		*index = newIndex(47, 0); /* small prime */
	}
	DIAG( printf("Store %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=addCollectionItem(*coll, rec);
	setIndexEntry(*index, slot, key1, key2);
}

/* find object with short integer keys */
struct genobject *find_object_i(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{								
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	short key1, key2;
	int slot;						
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((short *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((short *) FIELDADDR(rec, this->key2));
	if (!*coll) return NULL; 				
	DIAG( printf("Find %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=findIndexEntry(*index, key1, key2);		
	if (slot<0) return NULL;				
	return getCollectionItem(*coll, slot);		
}

/* store object with short integer keys */
void store_object_i(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	short key1, key2;
	int slot;
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((short *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((short *) FIELDADDR(rec, this->key2));
	if (!*coll) {
		*coll = newCollection(20,20);
		*index = newIndex(47, 0); /* small prime */
	}
	DIAG( printf("Store %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=addCollectionItem(*coll, rec);
	setIndexEntry(*index, slot, key1, key2);
}

/* find object with normal integer keys */
struct genobject *find_object_int(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{								
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	int key1, key2;
	int slot;						
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((int *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((int *) FIELDADDR(rec, this->key2));
	if (!*coll) return NULL; 				
	DIAG( printf("Find %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=findIndexEntry(*index, key1, key2);		
	if (slot<0) return NULL;				
	return getCollectionItem(*coll, slot);		
}

/* store object with normal integer keys */
void store_object_int(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	int key1, key2;
	int slot;
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((int *) FIELDADDR(rec, this->key1));
	if (this->key2<0)
		key2=0;
	else
		key2 = *((int *) FIELDADDR(rec, this->key2));
	if (!*coll) {
		*coll = newCollection(20,20);
		*index = newIndex(47, 0); /* small prime */
	}
	DIAG( printf("Store %s(%d,%d)\n", this->objdescr, key1, key2); )
	slot=addCollectionItem(*coll, rec);
	setIndexEntry(*index, slot, key1, key2);
}

/* find object with string keys */
struct genobject *find_object_s(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{								
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	char *key1;
	int slot;						
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((char **) FIELDADDR(rec, this->key1));
        if (!key1) {
          *((char **) FIELDADDR(rec, this->key1)) = (char *) malloc(1);
          key1 = *((char **) FIELDADDR(rec, this->key1));
          *key1 = '\0';
        }
	if (!*coll) return NULL; 				
	DIAG( printf("Find %s(%s)\n", this->objdescr, key1); )
	slot=findIndexEntryString(*index, key1);		
	if (slot<0) return NULL;				
	return getCollectionItem(*coll, slot);		
}

/* store object with string keys */
void store_object_s(self, edb, rec)
	void *self;
	Empire_Database *edb;
	struct genobject *rec;
{
	ObjectDescriptor *this;
	Collection **coll;
	Index **index;
	char *key1;
	int slot;
	this = (ObjectDescriptor *) self;
	coll  = (Collection **) FIELDADDR(edb, this->collection_offset);
	index = (Index **) FIELDADDR(edb, this->index_offset);
	key1 = *((char **) FIELDADDR(rec, this->key1));
        if (!key1) {
          *((char **) FIELDADDR(rec, this->key1)) = (char *) malloc(1);
          key1 = *((char **) FIELDADDR(rec, this->key1));
          *key1 = '\0';
        }
	if (!*coll) {
		*coll = newCollection(20,20);
		*index = newIndex(47, 0); /* small prime */
	}
	DIAG( printf("Store %s(%s)\n", this->objdescr, key1); )
	slot=addCollectionItem(*coll, rec);
	setIndexEntryString(*index, slot, key1);
}
