#include <struct.h>
#include <assert.h>
#include <stdio.h>
#include <ctype.h>
#include <malloc.h>
#include "edbproc.h"
#include "edbpriv.h"	/* private structures */
#include "misc.h"
#include "file.h"
#include "xy.h"
#include "nsc.h"
#include "edb.h"

#include "news.h"
#include "trade.h"
#include "loan.h"
#include "treaty.h"
#include "commodity.h"

#define HASNOVARS 0
#define HASVARS	1

extern struct castr sct_cf[], lnd_cf[], shp_cf[], pln_cf[], nuk_cf[], itm_cf[], prd_cf[];
extern struct castr bmpstr_ca[], mapstr_ca[], nat_ca[];
extern struct castr gameinfo_ca[];
extern struct castr news_ca[], trade_ca[],
		    loan_ca[], treaty_ca[],
		    commodity_ca[];

ObjectDescriptor objTable[] = {
	{ SCT_CLASS, "sctcls", "dchrstr", "sct_cf", "d_", HASNOVARS,
		sizeof(struct dchrstr), sct_cf,
		store_object_int, find_object_i,
		0, 0,
		fldoff( EDB_Str, sector_build),
		fldoff( EDB_Str, sector_build_index),
		fldoff( dchrstr, d_mnem),
		-1, -1, -1, -1 },
	{ SCT_OBJECT, "sect", "sctstr", "sect_ca", "sct_", HASVARS,
		sizeof(struct sctstr), sect_ca, 
		store_object_i, find_object_i,
		sctstr_putvar, sctstr_getvar,
		fldoff( EDB_Str, sector_data),
		fldoff( EDB_Str, sector_data_index),
		fldoff( sctstr, sct_x),
		fldoff( sctstr, sct_y),
		-1, -1, -1 },
	{ LND_CLASS, "lndcls", "lchrstr", "lnd_cf", "l_", HASVARS,
		sizeof(struct lchrstr), lnd_cf,
		store_object_s, find_object_s,
		lndcls_putvar, lndcls_getvar,
		fldoff( EDB_Str, land_build),
		fldoff( EDB_Str, land_build_index),
		fldoff( lchrstr, l_name),
		-1, -1, -1, -1 },
	{ LND_OBJECT,"land", "lndstr", "land_ca", "lnd_", HASVARS,
		sizeof(struct lndstr), land_ca,
		store_object_i, find_object_i,
		lndstr_putvar, lndstr_getvar,
		fldoff( EDB_Str, land_data),
                fldoff( EDB_Str, land_data_index),
		fldoff( lndstr, lnd_uid),
		-1,
		fldoff( EDB_Str, land_coord_index),
		fldoff( lndstr,  lnd_x),
		fldoff( lndstr,  lnd_y) },
	{ SHP_CLASS, "shpcls", "mchrstr", "shp_cf", "m_", HASNOVARS,
		sizeof(struct mchrstr), shp_cf,
		store_object_s, find_object_s,
		0, 0,
		fldoff( EDB_Str, ship_build),
		fldoff( EDB_Str, ship_build_index),
		fldoff( mchrstr, m_name),
		-1, -1, -1, -1 },
	{ SHP_OBJECT, "ship", "shpstr", "ship_ca", "shp_", HASVARS,
		sizeof(struct shpstr), ship_ca,
		store_object_i, find_object_i,
		shpstr_putvar, shpstr_getvar,
		fldoff( EDB_Str, ship_data),
                fldoff( EDB_Str, ship_data_index),
		fldoff( shpstr, shp_uid),
		-1,
		fldoff( EDB_Str, ship_coord_index),
		fldoff( shpstr,  shp_x),
		fldoff( shpstr,  shp_y) },
	{ PLN_CLASS,  "plncls", "plchrstr", "pln_cf", "pl_", HASNOVARS,
		sizeof(struct plchrstr), pln_cf,
		store_object_s, find_object_s,
		0, 0,
		fldoff( EDB_Str, plane_build),
		fldoff( EDB_Str, plane_build_index),
		fldoff( plchrstr, pl_name),
		-1, -1, -1, -1 },
	{ PLN_OBJECT, "plane", "plnstr", "plane_ca", "pln_", HASNOVARS,
		sizeof(struct plnstr), plane_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, plane_data),
                fldoff( EDB_Str, plane_data_index),
		fldoff( plnstr, pln_uid),
		-1,
		fldoff( EDB_Str, plane_coord_index),
		fldoff( plnstr,  pln_x),
		fldoff( plnstr,  pln_y) },
	{ NUK_CLASS,  "nukcls", "nchrstr", "nuk_cf", "n_", HASNOVARS,
		sizeof(struct nchrstr), nuk_cf,
		store_object_s, find_object_s,
		0, 0,
		fldoff( EDB_Str, nuke_build),
		fldoff( EDB_Str, nuke_build_index),
		fldoff( nchrstr, n_name),
		-1, -1, -1, -1 },
	{ NUK_OBJECT, "nuke", "nukstr", "nuke_ca", "nuk_", HASNOVARS,
		sizeof(struct nukstr), nuke_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, nuke_data),
                fldoff( EDB_Str, nuke_data_index),
		fldoff( nukstr, nuk_uid),
		-1,
		fldoff( EDB_Str, nuke_coord_index),
		fldoff( nukstr,  nuk_x),
		fldoff( nukstr,  nuk_y) },
	{ ITM_CLASS,  "itmcls", "ichrstr", "itm_cf", "i_", HASNOVARS,
		sizeof(struct ichrstr), itm_cf,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, item_class),
		fldoff( EDB_Str, item_class_index),
		fldoff( ichrstr, i_mnem),
		-1, -1, -1, -1 },
	{ PRD_CLASS,  "prdcls", "pchrstr", "prd_cf", "p_", HASVARS,
		sizeof(struct pchrstr), prd_cf,
		store_object_s, find_object_s,
		0, 0,
		fldoff( EDB_Str, product_class),
		fldoff( EDB_Str, product_class_index),
		fldoff( pchrstr, p_sname),
		-1, -1, -1, -1 },
	{ VER_CLASS,  "version", "gameinfo", "gameinfo_ca", "ver_", HASNOVARS,
		sizeof(struct gameinfo), gameinfo_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, game_data),
		fldoff( EDB_Str, game_data_index),
		fldoff( gameinfo, fld_MAXNOC),
		-1, -1, -1, -1 },

	{ BMP_OBJECT,  "bmap", "bmpstr", "bmpstr_ca", "bmp_", HASNOVARS,
		sizeof(struct bmpstr), bmpstr_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, bmap_data),
		fldoff( EDB_Str, bmap_data_index),
		fldoff( bmpstr, bmp_y),
		-1, -1, -1, -1 },

	{ MAP_OBJECT,  "map", "mapstr", "mapstr_ca", "map_", HASNOVARS,
		sizeof(struct mapstr), mapstr_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, map_data),
		fldoff( EDB_Str, map_data_index),
		fldoff( mapstr, map_x),
		fldoff( mapstr, map_y),
		-1, -1, -1 },

	{ NAT_OBJECT,  "nation", "natstr", "nat_ca", "nat_", HASNOVARS,
		sizeof(struct natstr), nat_ca,
		store_object_uchar, find_object_uchar,
		0, 0,
		fldoff( EDB_Str, country_data),
		fldoff( EDB_Str, country_data_index),
		fldoff( natstr, nat_cnum),
		-1, -1, -1, -1 },

	{ LOA_OBJECT,  "loan", "lonstr", "loan_ca", "l_", HASNOVARS,
		sizeof(struct lonstr), loan_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, loan_data),
		fldoff( EDB_Str, loan_data_index),
		fldoff( lonstr, l_uid),
		-1, -1, -1, -1 },

	{ TRD_OBJECT,  "trade", "trdstr", "trade_ca", "trd_", HASNOVARS,
		sizeof(struct trdstr), trade_ca,
		store_object_i, find_object_i,
		0, 0,
		fldoff( EDB_Str, trade_data),
		fldoff( EDB_Str, trade_data_index),
		fldoff( trdstr, trd_uid),
		-1, -1, -1, -1 },
	{ TRT_OBJECT,  "treaty", "trtstr", "treaty_ca", "trt_", HASNOVARS,
		sizeof(struct trdstr), trade_ca,
		store_object_nokey, find_object_nokey,
		0, 0,
		fldoff( EDB_Str, treaty_data),
		-1, -1, -1,
		-1, -1, -1 },
	{ NWS_OBJECT,  "news", "nwsstr", "news_ca", "nws_", HASNOVARS,
		sizeof(struct nwsstr), news_ca,
		store_object_nokey, find_object_nokey,
		0, 0,
		fldoff( EDB_Str, news_data),
		-1, -1, -1,
		-1, -1, -1 },
	
	{ COM_OBJECT,  "commodity", "comstr", "commodity_ca", "com_", HASNOVARS,
              sizeof(struct comstr), commodity_ca,
		store_object_nokey, find_object_nokey,
		0, 0,
		fldoff( EDB_Str, commod_data),
		-1, -1, -1,
		-1, -1, -1 },
	

	{ EF_BAD, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } };

